# Generated by Django 5.0 on 2024-01-05 15:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0008_dataservice"),
    ]

    operations = [
        migrations.AlterField(
            model_name="agent",
            name="type",
            field=models.CharField(blank=True, max_length=20),
        ),
    ]



from django.contrib import admin

# Register your models here.

from dcat.models import (
    Catalog,
    Dataset,
    Distribution,
    Agent,
    MediaType,
    LicenceDocument,
    DataTheme,
    Keyword,
)


class DatasetAdmin(admin.ModelAdmin):
    search_fields = ("title",)


class DistributionAdmin(admin.ModelAdmin):
    search_fields = ("title",)


class KeywordAdmin(admin.ModelAdmin):
    prepopulated_fields = {"slug": ["name"]}


class DataThemeAdmin(admin.ModelAdmin):
    search_fields = ("label",)


admin.site.register(Catalog)
admin.site.register(Dataset, DatasetAdmin)
admin.site.register(Distribution, DistributionAdmin)
admin.site.register(Agent)
admin.site.register(MediaType)
admin.site.register(LicenceDocument)
admin.site.register(DataTheme)
admin.site.register(Keyword, KeywordAdmin)



import json
import pathlib

from os import listdir

from django.core.exceptions import ObjectDoesNotExist
from django.core.files.base import ContentFile
from django.core.management.base import BaseCommand
from django.utils.text import slugify

from dcat.models import (
    Catalog,
    Dataset,
    Distribution,
    Agent,
    MediaType,
    LicenceDocument,
    DataTheme,
    Keyword,
)


class Command(BaseCommand):
    help = "Import data from a DCAT-US file provided by ckanext-datajson."

    def _get_content_file(self, dataset, distribution, datapath="data"):
        """Returns a ContentFile to be added to the django model.

        Requires the following structure:
        - datapath/
          - {dataset_identifier}/
            - {distribution_identifier}/
              - some-file.csv
        """
        file_folder = (
            f'{datapath}/{dataset.get("identifier")}/{distribution.get("identifier")}'
        )
        file = None
        try:
            local_file_name = listdir(file_folder)[0]
            file_path = f"{file_folder}/{local_file_name}"
            file = ContentFile(
                open(file_path, mode="rb").read(), name=distribution.get("fileName")
            )
        except IndexError:
            msg = f'{distribution.get("identifier")} folder does not have a file'
            self.stdout.write(self.style.ERROR(msg))
        return file

    def add_arguments(self, parser):
        parser.add_argument(
            "--file", type=open, help="Path to the data.json file", default="data.json"
        )
        parser.add_argument(
            "--datapath",
            type=pathlib.Path,
            help="Path to the data folder",
            default="data",
        )

    def handle(self, *args, **options):
        datapath = options.get("datapath")
        if not datapath.exists():
            msg = f"{datapath} path to data does not exist."
            self.stdout.write(self.style.ERROR(msg))
            return

        with options.get("file") as file:
            data = json.load(file)

        title = data.get("title")
        description = data.get("description")
        publisher, _ = Agent.objects.get_or_create(
            name=data.get("publisher").get("name"),
            mbox=data.get("publisher").get("mbox", ""),
        )
        catalog_licence, _ = LicenceDocument.objects.get_or_create(
            label=data.get("license")
        )
        catalog = Catalog.objects.create(
            title=title,
            description=description,
            publisher=publisher,
            licence=catalog_licence,
        )

        for theme in data.get("themeTaxonomy", []):
            theme_id = theme.get("id")
            theme_label = theme.get("label")
            theme_description = theme.get("description")

            theme_obj, _ = DataTheme.objects.get_or_create(
                code=theme_id,
                label=theme_label,
                description=theme_description,
            )
            catalog.themes.add(theme_obj)

        # Import Datasets
        datasets = data.get("dataset")
        for dataset in datasets:
            dataset_info = {}
            dataset_info["title"] = dataset.get("title")
            dataset_info["description"] = dataset.get("description")
            dataset_info["publisher"], _ = Agent.objects.get_or_create(
                name=dataset.get("publisher").get("name"),
                mbox=dataset.get("publisher").get("mbox", ""),
            )
            dataset_info["catalog"] = catalog
            dataset_created = Dataset.objects.create(**dataset_info)

            for theme in dataset.get("theme", []):
                try:
                    dataset_theme = DataTheme.objects.get(code=theme)
                except ObjectDoesNotExist:
                    msg = (
                        f"Theme of {dataset.get('identifier')} does not existed a theme"
                    )
                    self.stdout.write(self.style.WARNING(msg))
                dataset_created.themes.add(dataset_theme)

            for keyword in dataset.get("keyword", []):
                dataset_keyword, _ = Keyword.objects.get_or_create(
                    name=keyword, slug=slugify(keyword)
                    )
                dataset_created.keywords.add(dataset_keyword)

            # Import Distributions
            distributions = dataset.get("distribution", [])
            for distribution in distributions:
                distribution_info = {}
                distribution_info["dataset"] = dataset_created
                distribution_info["title"] = distribution.get("title")
                distribution_info["description"] = distribution.get("description", "")
                distribution_info["file"] = self._get_content_file(
                    dataset, distribution, datapath=options.get("datapath")
                )
                file_name = distribution.get("fileName")
                if not file_name:
                    # If the file name is not provided, the dataset is hosted
                    # in another portal. We add the download_url instead.
                    external_download = distribution.get("downloadURL")
                    if external_download:
                        distribution_info["external_download_url"] = distribution.get(
                            "downloadURL"
                        )
                    else:
                        msg = f'{distribution.get("identifier")} does not have a file name or a download url'
                        self.stdout.write(self.style.ERROR(msg))

                _format = distribution.get("format")
                if _format:
                    format, _ = MediaType.objects.get_or_create(extension=_format)
                    distribution_info["format"] = format

                _licence = distribution.get("license")
                if _licence:
                    licence, _ = LicenceDocument.objects.get_or_create(label=_licence)
                    distribution_info["licence"] = licence

                Distribution.objects.create(**distribution_info)

        self.stdout.write(self.style.SUCCESS("Data imported successfully"))



# Generated by Django 5.0 on 2023-12-20 23:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0003_catalog_homepage"),
    ]

    operations = [
        migrations.AlterField(
            model_name="catalog",
            name="themes",
            field=models.ManyToManyField(blank=True, to="dcat.datatheme"),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="description",
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="themes",
            field=models.ManyToManyField(blank=True, to="dcat.datatheme"),
        ),
    ]



""" Models for the DCAT application.

The models are based on the DCAT specification:
https://www.w3.org/TR/vocab-dcat-3/

Initial Mandatory/Recommended/Optional properties are based on the DCAT-AP
specification: https://semiceu.github.io/DCAT-AP/releases/3.0.0/

"""
import hashlib

from django.db import models


class Agent(models.Model):
    """Any entity carrying out actions with respect to the (Core) entities.

    Example: Publishers, Creators, etc.
    """

    # Mandatory properties
    name = models.CharField(max_length=255)

    # Recommended properties
    type = models.CharField(max_length=20, blank=True)

    # Optional properties
    mbox = models.EmailField(blank=True, null=True)

    def __str__(self):
        return self.name


class Catalog(models.Model):
    """A catalogue that hosts the Datasets or Data Services being described."""

    # Mandatory properties
    title = models.CharField(max_length=255)
    description = models.TextField()
    publisher = models.ForeignKey("Agent", on_delete=models.CASCADE)
    licence = models.ForeignKey("LicenceDocument", on_delete=models.SET_NULL, null=True)

    # Recommended properties
    themes = models.ManyToManyField("DataTheme", blank=True)
    homepage = models.URLField(blank=True)

    def __str__(self):
        return self.title


# class CatalogRecord(models.Model):
#     """A description of a Dataset's entry in the Catalogue.

#     It exists for catalogs where a distinction is made between metadata about a dataset or
#     service and metadata about the entry in the catalog about the dataset or
#     service. For example, the publication date property of the dataset reflects
#     the date when the information was originally made available by the
#     publishing agency, while the publication date of the catalog record is
#     the date when the dataset was added to the catalog.
#     """
#     pass


class Dataset(models.Model):
    """A conceptual entity that represents the information published."""

    # Mandatory properties
    title = models.CharField(max_length=255)
    catalog = models.ForeignKey("Catalog", on_delete=models.CASCADE)

    # Recommended properties
    description = models.TextField(blank=True)
    publisher = models.ForeignKey("Agent", on_delete=models.SET_NULL, null=True)
    themes = models.ManyToManyField("DataTheme", blank=True)
    keywords = models.ManyToManyField("Keyword", blank=True)

    # Optional properties
    modified = models.DateField(blank=True, null=True)
    issued = models.DateField(blank=True, null=True)

    def __str__(self):
        return self.title


# class DatasetInSeries(models.Model):
#     """Capture the case when a Dataset is a member of a Dataset Series."""
#     pass


# class DatasetSeries(models.Model):
#     """Datasets that are published separately, but share some characteristics."""
#     pass


class Distribution(models.Model):
    """A physical embodiment of the Dataset in a particular format.

    Example: A CSV file, an RDF file, etc.
    """

    def _get_storage_path(instance, filename):
        """Return the storage path of the file.

        The OS can complain if we store thousands of files in
        the same directory. So I'm inventing something to avoid
        this problem.
        """
        return f"files/datasets/{instance.dataset.pk}/{filename}"

    # Mandatory properties
    @property
    def access_url(self):
        pass

    # Recomened properties
    title = models.CharField(max_length=255, blank=True)
    description = models.TextField(blank=True)
    file = models.FileField(upload_to=_get_storage_path)
    dataset = models.ForeignKey("Dataset", on_delete=models.CASCADE)
    format = models.ForeignKey(
        "MediaType", on_delete=models.SET_NULL, blank=True, null=True
    )
    licence = models.ForeignKey(
        "LicenceDocument", on_delete=models.SET_NULL, blank=True, null=True
    )
    external_download_url = models.URLField(blank=True, default="")

    # Optional properties
    checksum = models.OneToOneField(
        "Checksum", on_delete=models.SET_NULL, blank=True, null=True
    )

    @property
    def download_url(self):
        """Return the download url of the file.

        If the file is hosted in another portal, the download_url is provided
        in the distribution. Otherwise, the download_url is generated from the
        file field.
        """
        if self.external_download_url:
            return self.external_download_url
        return self.file.url

    def calculate_md5_checksum(self):
        """Calculates the md5 checksum of the file."""
        md5_hash = hashlib.md5()
        with self.file.open(mode="rb") as f:
            while chunk := f.read(4096):
                md5_hash.update(chunk)
        return md5_hash.hexdigest()

    def __str__(self):
        return self.title


class DataService(models.Model):
    """Operations that provides access to datasets or data processing functions.

    Example: APIs, Web Services, SPARQL endpoints, etc.
    """

    # Mandatory properties
    title = models.CharField(max_length=255)
    endpoint_url = models.URLField()
    catalog = models.ForeignKey("Catalog", on_delete=models.CASCADE)

    # Optional properties
    description = models.TextField(blank=True)
    format = models.ManyToManyField("MediaType", blank=True)
    licence = models.ForeignKey("LicenceDocument", on_delete=models.SET_NULL, null=True)


class MediaType(models.Model):
    """A set of media types from the DCAT-AP vocabulary."""

    extension = models.CharField(max_length=10)
    code = models.CharField(max_length=10, unique=True, blank=True, null=True)
    media_type = models.CharField(max_length=50, blank=True)
    description = models.TextField(blank=True)

    def __str__(self):
        return self.extension


class LicenceDocument(models.Model):
    """A set of licences from the DCAT-AP vocabulary."""

    # Recommended properties
    @property
    def type(self):
        return self.url_general or self.label

    label = models.CharField(max_length=255)
    code = models.CharField(max_length=10, unique=True, blank=True, null=True)
    url_general = models.URLField(blank=True, default="")
    url_document = models.URLField(blank=True, default="")

    def __str__(self):
        return self.label


class DataTheme(models.Model):
    """Themes used for dataset classification.

    This is the mandatory controlled vocabulary for
    the themeTaxonomy field of Catalogue. (As defined
    by DCAT-AP).

    https://op.europa.eu/s/y52L
    """

    code = models.CharField(max_length=255, unique=True)
    label = models.CharField(max_length=255)
    description = models.TextField(blank=True, default="")

    def __str__(self):
        return self.label


class Checksum(models.Model):
    """A value that allows the contents of a file to be authenticated."""

    checksum_value = models.CharField(max_length=255)
    algorithm = models.CharField(max_length=10)

    def __str__(self):
        return f"{self.checksum_value} ({self.algorithm})"


class Keyword(models.Model):
    """A keyword or tag describing the Dataset."""

    name = models.CharField(max_length=50)
    slug = models.SlugField()

    def __str__(self):
        return self.name



import os
import xml.etree.ElementTree as ET

from django.core.management.base import BaseCommand
from dcat.models import MediaType


class Command(BaseCommand):
    help = "Import the EU's File types vocabulary into the database."

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **options):
        if not os.path.isfile("filetypes.xml"):
            self.stdout.write(
                self.style.ERROR(
                    "filetypes.xml does not exist. Please download it from https://op.europa.eu/s/y52f."
                )
            )
            return

        tree = ET.parse("filetypes.xml")
        root = tree.getroot()
        for record in root:
            authority_code = record.find("authority-code").text
            try:
                file_extension = record.find("file-extension").text
            except:
                self.stdout.write(
                    self.style.ERROR(
                        f"{authority_code} does not have a file extension. Skipping it."
                    )
                )
                continue
            internet_media_type = record.find("internet-media-type").text
            description = record.find("sources/source/description").text

            MediaType.objects.create(
                code=authority_code,
                extension=file_extension,
                media_type=internet_media_type,
                description=description,
            )

        total_types = MediaType.objects.count()
        self.stdout.write(
            self.style.SUCCESS(f"Successfully imported {total_types} filetypes.")
        )



# Generated by Django 5.0 on 2023-12-18 20:49

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0001_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="catalog",
            old_name="license",
            new_name="licence",
        ),
        migrations.RenameField(
            model_name="distribution",
            old_name="license",
            new_name="licence",
        ),
    ]



# Generated by Django 5.0 on 2023-12-21 17:02

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0005_keyword_dataset_keywords"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datatheme",
            name="code",
            field=models.CharField(max_length=255, unique=True),
        ),
    ]
